package data.scripts.world;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.FactionAPI;
import com.fs.starfarer.api.campaign.RepLevel;
import com.fs.starfarer.api.campaign.SectorAPI;
import com.fs.starfarer.api.impl.campaign.ids.Factions;
import com.fs.starfarer.api.impl.campaign.procgen.Constellation;
import static com.fs.starfarer.api.impl.campaign.procgen.SectorProcGen.CELL_SIZE;
import static com.fs.starfarer.api.impl.campaign.procgen.SectorProcGen.CONSTELLATION_CELLS;
import static com.fs.starfarer.api.impl.campaign.procgen.SectorProcGen.blotOut;
import com.fs.starfarer.api.impl.campaign.procgen.StarAge;
import com.fs.starfarer.api.impl.campaign.procgen.StarSystemGenerator;
import com.fs.starfarer.api.impl.campaign.procgen.themes.ThemeGenContext;
import com.fs.starfarer.api.impl.campaign.shared.SharedData;
import com.fs.starfarer.api.util.Pair;
import com.fs.starfarer.api.util.WeightedRandomPicker;
import data.scripts.campaign.procgen.themes.JunkPiratesAnarchistThemeGenerator;

import data.scripts.world.systems.Brehinni;
import data.scripts.world.systems.Canis;
//import data.scripts.world.systems.York;
import data.scripts.world.systems.Ursulo;
import java.util.ArrayList;
import java.util.List;
import org.lwjgl.util.vector.Vector2f;

public class JunkGen {

	public void generate(SectorAPI sector) {

        SharedData.getData().getPersonBountyEventData().addParticipatingFaction("pack");  
        SharedData.getData().getPersonBountyEventData().addParticipatingFaction("syndicate_asp");
        
        	float w = Global.getSettings().getFloat("sectorWidth");
		float h = Global.getSettings().getFloat("sectorHeight");
                
                int cellsWide = (int) (w / CELL_SIZE);
		int cellsHigh = (int) (h / CELL_SIZE);
                
		boolean [][] cells = new boolean [cellsWide][cellsHigh];

		int vPad = CONSTELLATION_CELLS / 2;
		int hPad = CONSTELLATION_CELLS / 2;
//		if (small) {
//			hPad = (int) (31000 / CELL_SIZE); 
//			vPad = (int) (19000 / CELL_SIZE); 
//		}
		
		for (int i = 0; i < cells.length; i++) {
			for (int j = 0; j < cells[0].length; j++) {
				if (i <= hPad || j <= vPad || i >= cellsWide - hPad || j >= cellsHigh - vPad) {
					cells[i][j] = true;
				}
			}
		}
        
                ThemeGenContext context = new ThemeGenContext();
                int count = 20;
                
                /*List<Constellation> c = new ArrayList<Constellation>();
		for (StarSystemAPI system : Global.getSector().getStarSystems()) {
			if (system.getConstellation() != null) {
                            if (c.contains(system.getConstellation())) {continue;
                            
                            }
                            c.add(system.getConstellation());
                        }
		}*/
                
                List<Constellation> constellations = new ArrayList();
		
                for (int k = 0; k < count; k++) {
			WeightedRandomPicker<Pair<Integer, Integer>> picker = new WeightedRandomPicker(StarSystemGenerator.random);
			for (int i = 0; i < cells.length; i++) {
				for (int j = 0; j < cells[0].length; j++) {
					if (cells[i][j]) continue;
					
					Pair<Integer, Integer> p = new Pair(i, j);
					picker.add(p);
				}
			}
			
			Pair<Integer, Integer> pick = picker.pick();
			if (pick == null) continue;
			
			blotOut(cells, pick.one, pick.two, CONSTELLATION_CELLS);
			
			float x = pick.one * CELL_SIZE - w / 2f;
			float y = pick.two * CELL_SIZE - h / 2f;
			
			StarSystemGenerator.CustomConstellationParams params = new StarSystemGenerator.CustomConstellationParams(StarAge.ANY);
			//if (!custom.isEmpty()) params = custom.remove(0);
			
			StarAge age = StarAge.ANY;
			if (age == StarAge.ANY) {
//				if (x < -w/6f) {
//					age = StarAge.YOUNG;
//				} else if (x > w/6f) {
//					age = StarAge.OLD;
//				} else {
//					age = StarAge.AVERAGE;
//				}
				WeightedRandomPicker<StarAge> agePicker = new WeightedRandomPicker(StarSystemGenerator.random);
				agePicker.add(StarAge.YOUNG);
				agePicker.add(StarAge.AVERAGE);
				agePicker.add(StarAge.OLD);
				age = agePicker.pick();
			}
			
			params.age = age;
			
			params.location = new Vector2f(x, y);
			Constellation c = new StarSystemGenerator(params).generate();
			constellations.add(c);
			
//			progress.render("Generating constellations...", 0.1f + 0.8f * (float)k / (float) count);
		}
                context.constellations = constellations;
                
                JunkPiratesAnarchistThemeGenerator face = new JunkPiratesAnarchistThemeGenerator();
                System.out.println("About to step in to ANARCHY");
                face.generateForSector(context, 3.0f);
        
		new Brehinni().generate(sector);
		new Canis().generate(sector);
		//new York().generate(sector);
		new Ursulo().generate(sector);

		initFactionRelationships(sector);
                                   
	}
        
    public static void initFactionRelationships(SectorAPI sector) {
        FactionAPI hegemony = sector.getFaction(Factions.HEGEMONY);
        FactionAPI tritachyon = sector.getFaction(Factions.TRITACHYON);
        FactionAPI pirates = sector.getFaction(Factions.PIRATES);
        FactionAPI independent = sector.getFaction(Factions.INDEPENDENT);
        FactionAPI kol = sector.getFaction(Factions.KOL);
        FactionAPI church = sector.getFaction(Factions.LUDDIC_CHURCH);
        FactionAPI path = sector.getFaction(Factions.LUDDIC_PATH);
        FactionAPI player = sector.getFaction(Factions.PLAYER);
        FactionAPI diktat = sector.getFaction(Factions.DIKTAT);
        FactionAPI junk = sector.getFaction("junk_pirates");
        FactionAPI pack = sector.getFaction("pack");
        FactionAPI asp = sector.getFaction("syndicate_asp");
        FactionAPI junkboys = sector.getFaction("junk_pirates_junkboys");
        FactionAPI technicians = sector.getFaction("junk_pirates_technicians");
        FactionAPI hounds = sector.getFaction("junk_pirates_hounds");
        FactionAPI league = sector.getFaction(Factions.PERSEAN);
        FactionAPI remnants = sector.getFaction(Factions.REMNANTS);
        
        //modded factions  
        FactionAPI sra = sector.getFaction("shadow_industry");  
        FactionAPI pirateAnar = sector.getFaction("pirateAnar");  
        FactionAPI mayorate = sector.getFaction("mayorate");  
        FactionAPI ice = sector.getFaction("sun_ice");  
        FactionAPI ici = sector.getFaction("sun_ici");  
        FactionAPI citadel = sector.getFaction("citadeldefenders");  
        FactionAPI II = sector.getFaction("interstellarimperium");
        FactionAPI exi = sector.getFaction("exigency");
        FactionAPI brdy = sector.getFaction("blackrock_driveyards");
        //FactionAPI lions = sector.getFaction("lions_guard");
        FactionAPI ahr = sector.getFaction("exipirated");
        FactionAPI pbc = sector.getFaction("pbc");
        FactionAPI sixieme = sector.getFaction("6eme_bureau");
        FactionAPI dme = sector.getFaction("dassault_mikoyan");
        //FactionAPI use = sector.getFaction("united_security");
        FactionAPI bb = sector.getFaction("blade_breakers");
        FactionAPI da = sector.getFaction("diableavionics");
        FactionAPI gmda = sector.getFaction("gmda");
        FactionAPI ars = sector.getFaction("al_ars");
        FactionAPI metel = sector.getFaction("metelson");
        FactionAPI ora = sector.getFaction("ORA");
        FactionAPI scy = sector.getFaction("SCY");
        FactionAPI hmi = sector.getFaction("HMI");
//        FactionAPI draco = sector.getFaction("draco");
//        FactionAPI fang = sector.getFaction("fang");
//        FactionAPI cartel = sector.getFaction("the_cartel");
        FactionAPI nullorder = sector.getFaction("nullorder");
        FactionAPI sylphon = sector.getFaction("sylphon");
        FactionAPI templar = sector.getFaction("templars");
//        FactionAPI coal = sector.getFaction("Coalition");
        FactionAPI cabal = sector.getFaction("cabal");
        FactionAPI maya = sector.getFaction("mayasura");
        FactionAPI sad = sector.getFaction("sad");
        FactionAPI thi = sector.getFaction("tiandong");
        FactionAPI kadur = sector.getFaction("kadur_remnant");

  
        if (sra != null) {  
            junk.setRelationship(sra.getId(), RepLevel.HOSTILE); // too damn nice for us
            asp.setRelationship(sra.getId(), RepLevel.WELCOMING); // we work together. They play nice.
            pack.setRelationship(sra.getId(), RepLevel.FAVORABLE); // I mean, what the fuck is going on? What day is it?
        }
        if (kadur != null) {  
            junk.setRelationship(kadur.getId(), RepLevel.FAVORABLE); // Yeah baby. Good guys looking to stick it to the man.
            asp.setRelationship(kadur.getId(), RepLevel.SUSPICIOUS); // Well; I'm not about to throw my lot in with these upstarts now, am I?
            pack.setRelationship(kadur.getId(), RepLevel.FAVORABLE); // Freedom and love and unity through suffering.
        }
        if (thi != null) {  
            junk.setRelationship(thi.getId(), RepLevel.HOSTILE); // Yeah; I'm not sure they like what we do. Fair enough.
            asp.setRelationship(thi.getId(), RepLevel.FAVORABLE); // Integrity in all the right places.
            pack.setRelationship(thi.getId(), RepLevel.FAVORABLE); // Honorable
        }
        if (brdy != null) {  
            junk.setRelationship(brdy.getId(), RepLevel.SUSPICIOUS); // don't trust these fuckers; but admire them
            asp.setRelationship(brdy.getId(), RepLevel.HOSTILE); // Bad for our business and too damn switched on.
            pack.setRelationship(brdy.getId(), RepLevel.NEUTRAL); // They seem pretty nice.
        }
        if (sixieme != null) {  
            junk.setRelationship(sixieme.getId(), RepLevel.HOSTILE); // I don't think they like us.
            asp.setRelationship(sixieme.getId(), RepLevel.WELCOMING); // Nice guys always finish first. Let's finish first together.
            pack.setRelationship(sixieme.getId(), RepLevel.WELCOMING); // These guys seem really friendly.
        }
        if (dme != null) {  
            junk.setRelationship(dme.getId(), RepLevel.SUSPICIOUS); // I'm sure they are up to something
            asp.setRelationship(dme.getId(), RepLevel.WELCOMING); // I'm sure we can make some money off these fellas
            pack.setRelationship(dme.getId(), RepLevel.NEUTRAL); // I'm sure I left the oven on
        }
        if (bb != null) {  
            junk.setRelationship(bb.getId(), RepLevel.HOSTILE); // So amazingly cool. But so dangerous.
            asp.setRelationship(bb.getId(), RepLevel.HOSTILE); // A recipe for disaster.
            pack.setRelationship(bb.getId(), RepLevel.HOSTILE); // Okay we're not THAT dumb.
        }
        if (da != null) {  
            junk.setRelationship(da.getId(), RepLevel.INHOSPITABLE); // Freedom is important to us.
            asp.setRelationship(da.getId(), RepLevel.SUSPICIOUS); // We really want to do business with these guys; but they just don't come to the party.
            pack.setRelationship(da.getId(), RepLevel.INHOSPITABLE); // A bit heavy. Like what's with all the population domination guys?
        }
        if (gmda != null) {  
            junk.setRelationship(gmda.getId(), RepLevel.NEUTRAL); // We tolerate them.
            asp.setRelationship(gmda.getId(), RepLevel.FAVORABLE); // As long as they don't police too close to us; they are okay
            pack.setRelationship(gmda.getId(), RepLevel.FAVORABLE); // Definitely a positive force. Also I really like Ainsley Harriot.
        }
        if (ars != null) {  
            junk.setRelationship(ars.getId(), RepLevel.HOSTILE); // Worth shooting on sight. Decent tech to steal; and no-one should mind.
            asp.setRelationship(ars.getId(), RepLevel.INHOSPITABLE); // Uncooperative and stubborn.
            pack.setRelationship(ars.getId(), RepLevel.INHOSPITABLE); // Is it me? Should I try being different?
        }
        if (metel != null) {  
            junk.setRelationship(metel.getId(), RepLevel.NEUTRAL); // Top buzzers; the little guy just trying to get by
            asp.setRelationship(metel.getId(), RepLevel.WELCOMING); // Hard work is easy to exploit for monetary gain. Thumbs up guys; keep the sector working.
            pack.setRelationship(metel.getId(), RepLevel.FAVORABLE); // Positive encounters; they seemed helpful and didn't try to wedgy us like the last nasty boys.
        }
        if (ora != null) {  
            junk.setRelationship(ora.getId(), RepLevel.HOSTILE); // So beautiful. I want their ships for my own nefarious means.
            asp.setRelationship(ora.getId(), RepLevel.FAVORABLE); // We don't have aligned ideals. But we are willing to get on.
            pack.setRelationship(ora.getId(), RepLevel.WELCOMING); // I think these fellas might be just what we have been looking for
        }
        if (hmi != null) {  
            junk.setRelationship(hmi.getId(), RepLevel.WELCOMING); // These fellas are nuts. We like them.
            asp.setRelationship(hmi.getId(), RepLevel.SUSPICIOUS); // We try and stay away from them. Liabilities; at best.
            pack.setRelationship(hmi.getId(), RepLevel.NEUTRAL); // Uhhhh ... huh? Who?
        }
//        if (draco != null) {  
//            junk.setRelationship(draco.getId(), RepLevel.HOSTILE);
//            asp.setRelationship(draco.getId(), RepLevel.WELCOMING);
//            pack.setRelationship(draco.getId(), RepLevel.NEUTRAL);
//        }
//        if (fang != null) {  
//            junk.setRelationship(fang.getId(), RepLevel.HOSTILE);
//            asp.setRelationship(fang.getId(), RepLevel.WELCOMING);
//            pack.setRelationship(fang.getId(), RepLevel.NEUTRAL);
//        }
//        if (cartel != null) {  
//            junk.setRelationship(cartel.getId(), RepLevel.HOSTILE);
//            asp.setRelationship(cartel.getId(), RepLevel.WELCOMING);
//            pack.setRelationship(cartel.getId(), RepLevel.NEUTRAL);
//        }
        if (nullorder != null) {  
            junk.setRelationship(nullorder.getId(), RepLevel.HOSTILE); // You know that girl you like who constantly punches you in the face?
            asp.setRelationship(nullorder.getId(), RepLevel.HOSTILE); // Simple economics - not good for business or reputation.
            pack.setRelationship(nullorder.getId(), RepLevel.HOSTILE); // What ... did you ... did you see that? Was it purple?
        }
        if (sylphon != null) {  
            junk.setRelationship(sylphon.getId(), RepLevel.WELCOMING); // You know that girl you like who you think wants to punch you in the face, but doesn't?
            asp.setRelationship(sylphon.getId(), RepLevel.NEUTRAL); // We can just about maintain the relationship without those Hedge Normies getting too bothered. Too bloody smart, though.
            pack.setRelationship(sylphon.getId(), RepLevel.FAVORABLE); // I don't understand this fuss. AIs are people too, right?
        }
        if (templar != null) {  
            junk.setRelationship(templar.getId(), RepLevel.VENGEFUL); // WEEEEE!!!
            asp.setRelationship(templar.getId(), RepLevel.VENGEFUL); // Fuckers
            pack.setRelationship(templar.getId(), RepLevel.VENGEFUL); // *melts*
        }
//        if (coal != null) {  
//            junk.setRelationship(coal.getId(), RepLevel.HOSTILE);
//            asp.setRelationship(coal.getId(), RepLevel.WELCOMING);
//            pack.setRelationship(coal.getId(), RepLevel.NEUTRAL);
//        }
        if (cabal != null) {  
            junk.setRelationship(cabal.getId(), RepLevel.HOSTILE); // nasty bits of work
            asp.setRelationship(cabal.getId(), RepLevel.WELCOMING); // let's not kid ourselves here .. we have common ground
            pack.setRelationship(cabal.getId(), RepLevel.INHOSPITABLE); // They've already beat us down repeatedly
        }
        if (scy != null) {  
            junk.setRelationship(scy.getId(), RepLevel.NEUTRAL); // Not great, but not Hegemony.
            asp.setRelationship(scy.getId(), RepLevel.WELCOMING); // A lot of mutual backscratching opportunities
            pack.setRelationship(scy.getId(), RepLevel.NEUTRAL); // They kinda don't even know who we are, I think.
        }
        if (pbc != null) {  
            junk.setRelationship(pbc.getId(), RepLevel.HOSTILE);
            asp.setRelationship(pbc.getId(), RepLevel.WELCOMING);
            pack.setRelationship(pbc.getId(), RepLevel.NEUTRAL);
        }  
        if (pirateAnar != null) {  
            junk.setRelationship(pirateAnar.getId(), RepLevel.FAVORABLE);
            asp.setRelationship(pirateAnar.getId(), RepLevel.HOSTILE); 
            pack.setRelationship(pirateAnar.getId(), RepLevel.SUSPICIOUS); 
        }  
        if (mayorate != null) {  
            junk.setRelationship(mayorate.getId(), RepLevel.SUSPICIOUS);
            asp.setRelationship(mayorate.getId(), RepLevel.HOSTILE);
            pack.setRelationship(mayorate.getId(), RepLevel.NEUTRAL);
        }  
        if (ice != null) {  
            junk.setRelationship(ice.getId(), RepLevel.NEUTRAL);
            asp.setRelationship(ice.getId(), RepLevel.NEUTRAL);
            pack.setRelationship(ice.getId(), RepLevel.NEUTRAL);
        }  
        if (ici != null) {  
            junk.setRelationship(ici.getId(), RepLevel.FAVORABLE);
            asp.setRelationship(ici.getId(), RepLevel.SUSPICIOUS);
            pack.setRelationship(ici.getId(), RepLevel.NEUTRAL);
        }  
        if (citadel != null) {  
            junk.setRelationship(citadel.getId(), RepLevel.NEUTRAL);
            asp.setRelationship(citadel.getId(), RepLevel.NEUTRAL);
            pack.setRelationship(citadel.getId(), RepLevel.FAVORABLE);
        }  
          
        if (II != null) {  
            junk.setRelationship(II.getId(), RepLevel.HOSTILE); // Bloody Hegemony by another name. Too big and too bloody ordered.
            asp.setRelationship(II.getId(), RepLevel.SUSPICIOUS); // Competition.
            pack.setRelationship(II.getId(), RepLevel.NEUTRAL); // Decent enough fellas; they leave us alone.
        }  
        
        if (ahr != null) {  
            junk.setRelationship(ahr.getId(), RepLevel.FAVORABLE);
            asp.setRelationship(ahr.getId(), RepLevel.INHOSPITABLE); 
            pack.setRelationship(ahr.getId(), RepLevel.SUSPICIOUS); 
        } 
        
        if (exi != null) {  
            junk.setRelationship(exi.getId(), RepLevel.HOSTILE);
            asp.setRelationship(exi.getId(), RepLevel.HOSTILE); 
            pack.setRelationship(exi.getId(), RepLevel.SUSPICIOUS); 
        } 
        
        player.setRelationship(junk.getId(), 0);
        player.setRelationship(pack.getId(), 0);
        player.setRelationship(asp.getId(), 0);
        
        junk.setRelationship(hegemony.getId(), RepLevel.HOSTILE);
        junk.setRelationship(path.getId(), RepLevel.HOSTILE);
        junk.setRelationship(church.getId(), RepLevel.HOSTILE);
        junk.setRelationship(pirates.getId(), RepLevel.FAVORABLE);
        junk.setRelationship(diktat.getId(), RepLevel.HOSTILE);
        junk.setRelationship(tritachyon.getId(), RepLevel.HOSTILE);
        junk.setRelationship(independent.getId(), RepLevel.NEUTRAL);
        junk.setRelationship(asp.getId(), RepLevel.HOSTILE);
        junk.setRelationship(pack.getId(), RepLevel.FAVORABLE);
        junk.setRelationship(league.getId(), RepLevel.NEUTRAL); // Common borders; League remain more open than Heg
        
        List<FactionAPI> factionList = sector.getAllFactions();
        factionList.remove(junkboys);
        factionList.remove(technicians);
        factionList.remove(hounds);
        for (FactionAPI faction : factionList) {
        junkboys.setRelationship(faction.getId(), RepLevel.HOSTILE);
        technicians.setRelationship(faction.getId(), RepLevel.HOSTILE);
        hounds.setRelationship(faction.getId(), RepLevel.HOSTILE);
        }
        junkboys.setRelationship("player", RepLevel.HOSTILE);
        technicians.setRelationship("player", RepLevel.HOSTILE);
        hounds.setRelationship("player", RepLevel.HOSTILE);
        
        tritachyon.setRelationship(junk.getId(), RepLevel.HOSTILE); // We are not a tech mine
        hegemony.setRelationship(junk.getId(), RepLevel.HOSTILE); // Villainous scum
        pirates.setRelationship(junk.getId(), RepLevel.FAVORABLE); // He he these fellas are funny
        kol.setRelationship(junk.getId(), RepLevel.HOSTILE); // IN LUDDS NAME BEGONE
        league.setRelationship(junk.getId(), RepLevel.NEUTRAL); // Kinda misunderstood
        independent.setRelationship(junk.getId(), RepLevel.NEUTRAL); // Just like us; if more chaotic
        church.setRelationship(junk.getId(), RepLevel.NEUTRAL);
        path.setRelationship(junk.getId(), RepLevel.HOSTILE); // tech lovin heathens
        diktat.setRelationship(junk.getId(), RepLevel.HOSTILE); // Get outta our sector and hands off my synchrotron core
        

        pack.setRelationship(hegemony.getId(), RepLevel.SUSPICIOUS);
        pack.setRelationship(pirates.getId(), RepLevel.NEUTRAL);
        pack.setRelationship(church.getId(), RepLevel.NEUTRAL);
        pack.setRelationship(path.getId(), RepLevel.HOSTILE);
        pack.setRelationship(diktat.getId(), RepLevel.SUSPICIOUS);
        pack.setRelationship(tritachyon.getId(), RepLevel.SUSPICIOUS);
        pack.setRelationship(independent.getId(), RepLevel.FAVORABLE);
        pack.setRelationship(junk.getId(), RepLevel.FAVORABLE);
        pack.setRelationship(asp.getId(), RepLevel.NEUTRAL);
        pack.setRelationship(league.getId(), RepLevel.WELCOMING);
        
        tritachyon.setRelationship(pack.getId(), RepLevel.SUSPICIOUS);
        hegemony.setRelationship(pack.getId(), RepLevel.NEUTRAL);
        pirates.setRelationship(pack.getId(), RepLevel.HOSTILE);
        kol.setRelationship(pack.getId(), RepLevel.NEUTRAL);
        league.setRelationship(pack.getId(), RepLevel.WELCOMING);
        diktat.setRelationship(pack.getId(), RepLevel.SUSPICIOUS);
        independent.setRelationship(pack.getId(), RepLevel.FAVORABLE);
        church.setRelationship(pack.getId(), RepLevel.SUSPICIOUS);
        path.setRelationship(pack.getId(), RepLevel.SUSPICIOUS);

        asp.setRelationship(hegemony.getId(), RepLevel.FAVORABLE);
        asp.setRelationship(pirates.getId(), RepLevel.HOSTILE);
        asp.setRelationship(church.getId(), RepLevel.FAVORABLE);
        asp.setRelationship(path.getId(), RepLevel.HOSTILE);
        asp.setRelationship(diktat.getId(), RepLevel.FAVORABLE);
        asp.setRelationship(tritachyon.getId(), RepLevel.WELCOMING);
        asp.setRelationship(independent.getId(), RepLevel.WELCOMING);
        asp.setRelationship(pack.getId(), RepLevel.NEUTRAL);
        asp.setRelationship(league.getId(), RepLevel.HOSTILE); // it just aint working. They want to cut us off
        asp.setRelationship(junk.getId(), RepLevel.HOSTILE);

	pirates.setRelationship(asp.getId(), RepLevel.HOSTILE);
        tritachyon.setRelationship(pack.getId(), RepLevel.WELCOMING);
        hegemony.setRelationship(pack.getId(), RepLevel.FAVORABLE);
        kol.setRelationship(pack.getId(), RepLevel.NEUTRAL);
        diktat.setRelationship(pack.getId(), RepLevel.NEUTRAL);
	league.setRelationship(asp.getId(), RepLevel.HOSTILE);
	church.setRelationship(asp.getId(), RepLevel.FAVORABLE);
	path.setRelationship(asp.getId(), RepLevel.HOSTILE);
	independent.setRelationship(asp.getId(), RepLevel.WELCOMING);
    }
}
