package data.scripts.world.systems;

import java.awt.Color;

import com.fs.starfarer.api.impl.campaign.procgen.StarAge;
import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.JumpPointAPI;
import com.fs.starfarer.api.campaign.LocationAPI;
import com.fs.starfarer.api.campaign.OrbitAPI;
import com.fs.starfarer.api.campaign.PlanetAPI;
import com.fs.starfarer.api.campaign.SectorAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.StarSystemAPI;
//import com.fs.starfarer.api.campaign.CargoAPI.CrewXPLevel;
import com.fs.starfarer.api.impl.campaign.DerelictShipEntityPlugin;
import com.fs.starfarer.api.impl.campaign.ids.Entities;
import com.fs.starfarer.api.impl.campaign.ids.Factions;
import com.fs.starfarer.api.impl.campaign.ids.Terrain;
import com.fs.starfarer.api.impl.campaign.terrain.BaseRingTerrain.RingParams;
import com.fs.starfarer.api.impl.campaign.terrain.AsteroidFieldTerrainPlugin.AsteroidFieldParams;
// import com.fs.starfarer.api.impl.campaign.terrain.PlumeTerrainPlugin.PlumeParams;
import com.fs.starfarer.api.impl.campaign.terrain.HyperspaceTerrainPlugin;
import com.fs.starfarer.api.impl.campaign.procgen.NebulaEditor;
import com.fs.starfarer.api.impl.campaign.procgen.themes.BaseThemeGenerator;
import com.fs.starfarer.api.impl.campaign.procgen.themes.SalvageSpecialAssigner;
import com.fs.starfarer.api.impl.campaign.rulecmd.salvage.special.ShipRecoverySpecial;
import com.fs.starfarer.api.util.Misc;

public class Brehinni {

	public void generate(SectorAPI sector) {

		StarSystemAPI system = sector.createStarSystem("Breh'Inni");
		LocationAPI hyper = Global.getSector().getHyperspace();

		system.setBackgroundTextureFilename("graphics/backgrounds/background5.jpg");

		SectorEntityToken brehinni_nebula = Misc.addNebulaFromPNG("data/campaign/terrain/brehinni_nebula.png",
				  0, 0, // center of nebula
				  system, // location to add to
				  "terrain", "nebula_blue", // "nebula_blue", // texture to use, uses xxx_map for map
				  4, 4, // number of cells in texture
                                  StarAge.YOUNG); 

		// create the star and generate the hyperspace anchor for this system
		PlanetAPI star = system.initStar("brehinni", // unique id for this star
										 "star_red_giant", // id in planets.json
										 800f, 400); 		// radius (in pixels at default zoom)

		system.setLightColor(new Color(255, 210, 200)); // light color in entire system, affects all entities

		/*
		 * addPlanet() parameters:
		 * 1. Unique id for this planet (or null to have it be autogenerated)
		 * 2. What the planet orbits (orbit is always circular)
		 * 3. Name
		 * 4. Planet type id in planets.json
		 * 5. Starting angle in orbit, i.e. 0 = to the right of the star
		 * 6. Planet radius, pixels at default zoom
		 * 7. Orbit radius, pixels at default zoom
		 * 8. Days it takes to complete an orbit. 1 day = 10 seconds.
		 */





		PlanetAPI bh1 = system.addPlanet("ear_burns", star, "Ear Burns", "rocky_metallic", 70, 120, 2000, 60);
		bh1.setFaction("independent");
		bh1.setCustomDescriptionId("planet_brehinniI");
		bh1.setInteractionImage("illustrations", "vacuum_colony");
		bh1.getSpec().setGlowTexture(Global.getSettings().getSpriteName("hab_glows", "barren"));
		bh1.getSpec().setGlowColor(new Color(245,255,250,255));
		bh1.applySpecChanges();

		PlanetAPI bh2 = system.addPlanet("back_spin", star, "Backspin", "gas_giant", 120, 240, 3500, 130);
		bh2.setCustomDescriptionId("planet_brehinniII");

		SectorEntityToken milStation = system.addCustomEntity("karkov_academy", "Karkov Academy", "station_karkov_academy", "junk_pirates");
		milStation.setCircularOrbitPointingDown(system.getEntityById("back_spin"), 90, 275, 45);
		milStation.setCustomDescriptionId("karkov_academy");
		milStation.setInteractionImage("illustrations", "karkov_academy");

		bh2.getSpec().setPlanetColor(new Color(195,195,215,255));
		bh2.getSpec().setAtmosphereColor(new Color(160,110,45,140));
		bh2.getSpec().setCloudColor(new Color(255,164,96,200));
		bh2.getSpec().setTilt(15);
		bh2.applySpecChanges();

		system.addRingBand(bh2, "misc", "rings_dust0", 256f, 2, Color.white, 256f, 250, 40f);
		system.addRingBand(bh2, "misc", "rings_dust0", 256f, 1, Color.white, 256f, 260, 60f);
		system.addRingBand(bh2, "misc", "rings_dust0", 256f, 3, Color.white, 256f, 270, 120f);

		SectorEntityToken ring = system.addTerrain(Terrain.RING, new RingParams(20 + 256, 260, null, "Backspin Debris Field"));
		ring.setCircularOrbit(bh2, 0, 0, 100);

		PlanetAPI bh3 = system.addPlanet("glory", star, "Glory", "planet_glory", 240, 150, 6300, 130);
		bh3.setFaction("junk_pirates");
		bh3.setCustomDescriptionId("planet_brehinniIII");
		bh3.getSpec().setGlowTexture(Global.getSettings().getSpriteName("hab_glows", "junk_pirates_glory"));
		bh3.getSpec().setGlowColor(new Color(245,255,250,255));
		bh3.setInteractionImage("illustrations", "glory_background");bh3.applySpecChanges();

		PlanetAPI bh4 = system.addPlanet("hope", star, "Hope", "gas_giant", 180, 260, 10000, 260);

		bh4.setCustomDescriptionId("planet_brehinniIV");
		bh4.getSpec().setPlanetColor(new Color(175,135,245,255));
		bh4.getSpec().setAtmosphereColor(new Color(160,110,45,140));
		bh4.getSpec().setCloudColor(new Color(195,195,215,200));
		bh4.getSpec().setTilt(20);
		bh4.applySpecChanges();

		system.addRingBand(bh4, "misc", "rings_dust0", 256f, 2, Color.white, 256f, 1100, 40f);
		system.addRingBand(bh4, "misc", "rings_ice0", 256f, 3, Color.white, 256f, 1150, 60f);
		system.addRingBand(bh4, "misc", "rings_dust0", 256f, 0, Color.white, 256f, 1300, 120f);

		ring = system.addTerrain(Terrain.RING, new RingParams(200 + 256, 1200, null, "Hope Rings"));
		ring.setCircularOrbit(bh4, 0, 0, 100);

		SectorEntityToken relay = system.addCustomEntity("listening_post", // unique id
				 "Listening Post", // name - if null, defaultName from custom_entities.json will be used
				 "comm_relay", // type of object, defined in custom_entities.json
				 "junk_pirates"); // faction
		relay.setCircularOrbit( system.getEntityById("brehinni"), 240, 10000, 260);

		JumpPointAPI jumpPoint = Global.getFactory().createJumpPoint("brehinni_gate", "Breh'Inni Gate");
		OrbitAPI orbit = Global.getFactory().createCircularOrbit(star, 120, 10000, 260);
		jumpPoint.setOrbit(orbit);
		//jumpPoint.setRelatedPlanet(bh4);
		jumpPoint.setStandardWormholeToHyperspaceVisual();
		system.addEntity(jumpPoint);

		PlanetAPI bh5 = system.addPlanet("nearly_there", star, "Nearly There", "rocky_ice", 290, 120, 12000, 340);
		PlanetAPI bh6 = system.addPlanet("too_far", star, "Too Far", "rocky_metallic", 280, 90, 15000, 340);

			SectorEntityToken bh5bh6debris = system.addTerrain(Terrain.ASTEROID_FIELD,
					new AsteroidFieldParams(
						500f, // min radius
						700f, // max radius
						20, // min asteroid count
						30, // max asteroid count
						4f, // min asteroid radius 
						16f, // max asteroid radius
						"N.O.M.D.E. Ejected Mass Field")); // null for default name

			bh5bh6debris.setCircularOrbit(star, 285f, 13500, 340);

		SectorEntityToken volStation = system.addCustomEntity("nomde_plume", "N.O.M.D.E.", "station_nomde_plume", "junk_pirates");
		volStation.setCircularOrbitPointingDown(system.getEntityById("nearly_there"), 90, 315, 125);
		volStation.setCustomDescriptionId("nomde_plume");
		volStation.setInteractionImage("illustrations", "pirate_station");
		//plume terrain; thickness, middle radius, related entity, dir, width, R, G, B
		//SectorEntityToken plume = system.addTerrain("plume", new PlumeParams(120f, 180f, volStation, 90f, 60f, 225, 165, 145));

		bh5.setCustomDescriptionId("planet_brehinniV");
		bh6.setCustomDescriptionId("planet_brehinniVI");

		bh6.getSpec().setPlanetColor(new Color(215,255,190,255));
		bh6.getSpec().setAtmosphereColor(new Color(160,110,45,140));
		bh6.getSpec().setCloudColor(new Color(175,135,245,200));
		bh6.getSpec().setTilt(10);
		bh6.applySpecChanges();

		PlanetAPI york4 = system.addPlanet("swanage", star, "Swanage", "gas_giant", 15, 300, 15000, 450);

		//york4.setCustomDescriptionId("planet_yorkIV");

		york4.getSpec().setPlanetColor(new Color(215,225,115,255));
		york4.getSpec().setAtmosphereColor(new Color(160,185,45,140));
		york4.getSpec().setCloudColor(new Color(195,155,115,200));
		york4.getSpec().setTilt(45);
		york4.applySpecChanges();

		PlanetAPI york4a = system.addPlanet("dartmouth", york4, "Dartmouth", "rocky_ice", 0, 55, 1000, 45);
		//york4a.getSpec().setGlowTexture(Global.getSettings().getSpriteName("hab_glows", "barren"));
		york4a.getSpec().setGlowColor(new Color(245,235,250,185));
		york4a.applySpecChanges();
		//york4a.setFaction("junk_pirates");

		PlanetAPI york4b = system.addPlanet("christchurch", york4, "Christchurch", "rocky_metallic", 180, 65, 1000, 45);
		york4b.getSpec().setGlowTexture(Global.getSettings().getSpriteName("hab_glows", "volturn"));
		york4b.getSpec().setGlowColor(new Color(245,255,250,125));
		york4b.applySpecChanges();
		york4b.setFaction("pack");

		york4a.setCustomDescriptionId("planet_yorkIVa");
		york4b.setCustomDescriptionId("planet_yorkIVb");

		SectorEntityToken spinerette = system.addCustomEntity("spinerette", "Spinerette", "junk_pirates_spinerette", "junk_pirates");
		spinerette.setCircularOrbitPointingDown(system.getEntityById("swanage"), 90, 1000, 45);
		//spinerette.setCustomDescriptionId("junk_pirates_spinerette");
		//spinerette.setInteractionImage("illustrations", "pirate_station");

		SectorEntityToken hypercube = system.addCustomEntity("hypercube", "Hypercube", "junk_pirates_hypercube", "junk_pirates");
		hypercube.setCircularOrbitPointingDown(system.getEntityById("swanage"), 270, 1000, 45);
		//spinerette.setCustomDescriptionId("junk_pirates_spinerette");
		//spinerette.setInteractionImage("illustrations", "pirate_station");

		//SectorEntityToken station = system.addOrbitalStation("diktat_cnc", a1, 45, 300, 50, "Command & Control", "sindrian_diktat");
		//initStationCargo(station);

		//SectorEntityToken station = system.addCustomEntity("diktat_cnc", "Command & Control", "station_side02", "sindrian_diktat");
		//station.setCircularOrbitPointingDown(system.getEntityById("sindria"), 45, 300, 50);
//		station.setCustomDescriptionId("station_ragnar");

		// example of using custom visuals below
//		a1.setCustomInteractionDialogImageVisual(new InteractionDialogImageVisual("illustrations", "hull_breach", 800, 800));
//		jumpPoint.setCustomInteractionDialogImageVisual(new InteractionDialogImageVisual("illustrations", "space_wreckage", 1200, 1200));
//		station.setCustomInteractionDialogImageVisual(new InteractionDialogImageVisual("illustrations", "cargo_loading", 1200, 1200));

		// generates hyperspace destinations for in-system jump points
		system.autogenerateHyperspaceJumpPoints(true, true);

                addDerelict(system, hypercube, "junk_pirates_tangerine_Hull", ShipRecoverySpecial.ShipCondition.AVERAGE, 75, true);

		/*
		DiktatPatrolSpawnPoint patrolSpawn = new DiktatPatrolSpawnPoint(sector, system, 5, 3, a1);
		system.addScript(patrolSpawn);
		for (int i = 0; i < 5; i++)
			patrolSpawn.spawnFleet();

		DiktatGarrisonSpawnPoint garrisonSpawn = new DiktatGarrisonSpawnPoint(sector, system, 30, 1, a1, a1);
		system.addScript(garrisonSpawn);
		garrisonSpawn.spawnFleet();


		system.addScript(new IndependentTraderSpawnPoint(sector, hyper, 1, 10, hyper.createToken(-6000, 2000), station));
		*/
                cleanup(system);
	}

    private void addDerelict(StarSystemAPI system, 
            SectorEntityToken focus, 
            String variantId, 
            ShipRecoverySpecial.ShipCondition condition, 
            float orbitRadius, 
            boolean recoverable) {
        DerelictShipEntityPlugin.DerelictShipData params = new DerelictShipEntityPlugin.DerelictShipData(new ShipRecoverySpecial.PerShipData(variantId, condition), false);
        SectorEntityToken ship = BaseThemeGenerator.addSalvageEntity(system, Entities.WRECK, Factions.NEUTRAL, params);
        ship.setDiscoverable(true);

        float orbitDays = orbitRadius / (10f + (float) Math.random() * 5f);
        ship.setCircularOrbit(focus, (float) Math.random() * 360f, orbitRadius, orbitDays);

        if (recoverable) {
            SalvageSpecialAssigner.ShipRecoverySpecialCreator creator = new SalvageSpecialAssigner.ShipRecoverySpecialCreator(null, 0, 0, false, null, null);
            Misc.setSalvageSpecial(ship, creator.createSpecial(ship, null));
        }
    }
        
    void cleanup(StarSystemAPI system){
        HyperspaceTerrainPlugin plugin = (HyperspaceTerrainPlugin) Misc.getHyperspaceTerrain().getPlugin();
	NebulaEditor editor = new NebulaEditor(plugin);        
        float minRadius = plugin.getTileSize() * 2f;
        
        float radius = system.getMaxRadiusInHyperspace();
        editor.clearArc(system.getLocation().x, system.getLocation().y, 0, radius + minRadius * 0.5f, 0, 360f);
        editor.clearArc(system.getLocation().x, system.getLocation().y, 0, radius + minRadius, 0, 360f, 0.25f);	     
//        editor.clearArc(system.getLocation().x, system.getLocation().y, 0, system.getMaxRadiusInHyperspace()*1.25f, 0, 360, 0.25f);
    }
}
