package data.scripts.world.systems;

import java.awt.Color;
import java.util.List;

import com.fs.starfarer.api.impl.campaign.procgen.StarAge;
import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.CargoAPI;
import com.fs.starfarer.api.campaign.JumpPointAPI;
import com.fs.starfarer.api.campaign.LocationAPI;
import com.fs.starfarer.api.campaign.OrbitAPI;
import com.fs.starfarer.api.campaign.PlanetAPI;
import com.fs.starfarer.api.campaign.SectorAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.StarSystemAPI;
//import com.fs.starfarer.api.campaign.CargoAPI.CrewXPLevel;
import com.fs.starfarer.api.fleet.FleetMemberType;
import com.fs.starfarer.api.impl.campaign.ids.Terrain;
import com.fs.starfarer.api.impl.campaign.procgen.NebulaEditor;
import com.fs.starfarer.api.impl.campaign.terrain.BaseRingTerrain.RingParams;
import com.fs.starfarer.api.impl.campaign.terrain.MagneticFieldTerrainPlugin.MagneticFieldParams;
import com.fs.starfarer.api.util.Misc;
import com.fs.starfarer.api.impl.campaign.terrain.HyperspaceTerrainPlugin;
import com.fs.starfarer.api.impl.campaign.procgen.NebulaEditor;

public class Canis {

	public void generate(SectorAPI sector) {
		
		StarSystemAPI system = sector.createStarSystem("Canis");
		LocationAPI hyper = Global.getSector().getHyperspace();
		
		system.setBackgroundTextureFilename("graphics/backgrounds/background3.jpg");
		
		SectorEntityToken canis_nebula = Misc.addNebulaFromPNG("data/campaign/terrain/canis_nebula.png",
				  0, 0, // center of nebula
				  system, // location to add to
				  "terrain", "nebula_blue", // "nebula_blue", // texture to use, uses xxx_map for map
				  4, 4, StarAge.YOUNG); // number of cells in texture

		// create the star and generate the hyperspace anchor for this system
		PlanetAPI star = system.initStar("canis", "star_red_dwarf", 600f, 200);
		
		system.setLightColor(new Color(255, 220, 180)); // light color in entire system, affects all entities
		
		/*
		 * addPlanet() parameters:
		 * 1. Unique id for this planet (or null to have it be autogenerated)
		 * 2. What the planet orbits (orbit is always circular)
		 * 3. Name
		 * 4. Planet type id in planets.json
		 * 5. Starting angle in orbit, i.e. 0 = to the right of the star
		 * 6. Planet radius, pixels at default zoom
		 * 7. Orbit radius, pixels at default zoom
		 * 8. Days it takes to complete an orbit. 1 day = 10 seconds.
		 */
		
		
		


		PlanetAPI canis1 = system.addPlanet("spot", star, "Spot", "rocky_metallic", 100, 120, 2000, 60);

		system.addRingBand(canis1, "misc", "rings_dust0", 256f, 0, Color.white, 256f, 400, 100f);
		system.addRingBand(canis1, "misc", "rings_asteroids0", 256f, 2, Color.white, 256f, 420, 80f);

		SectorEntityToken ring = system.addTerrain(Terrain.RING, new RingParams(20 + 256, 410, null, "Spot's Belt"));
		ring.setCircularOrbit(canis1, 0, 0, 100);

		SectorEntityToken pirateStation = system.addCustomEntity("jameson_orbital", "Jameson Orbital", "station_pirate_type", "pirates");
		pirateStation.setCircularOrbitPointingDown(system.getEntityById("spot"), 90, 450, 45);		
		pirateStation.setCustomDescriptionId("jameson_orbital");
		pirateStation.setInteractionImage("illustrations", "pirate_station");

		PlanetAPI canis2 = system.addPlanet("hooch", star, "Hooch", "rocky_ice", 120, 240, 4200, 180);
		SectorEntityToken hulk = system.addCustomEntity("hulk", "Decaying Hulk", "junk_pirates_abandoned_hulk", "neutral");
		hulk.setCircularOrbitPointingDown(system.getEntityById("hooch"), 270, 360, 45);		

		PlanetAPI canis3 = system.addPlanet("petra", star, "Petra", "gas_giant", 270, 250, 6800, 200);
		canis3.setFaction("pack");
		canis3.setCustomDescriptionId("planet_canisIII");
		canis3.setInteractionImage("illustrations", "petra_port");
		canis3.getSpec().setPlanetColor(new Color(140,195,140,255));
		canis3.getSpec().setAtmosphereColor(new Color(120,225,45,140));
		canis3.getSpec().setCloudColor(new Color(160,255,96,200));
		canis3.getSpec().setTilt(10);
		canis3.applySpecChanges();

		system.addRingBand(star, "misc", "rings_ice0", 256f, 3, Color.white, 256f, 8225, 200f);
		system.addRingBand(star, "misc", "rings_ice0", 256f, 2, Color.white, 256f, 8300, 250f);
		system.addAsteroidBelt(star, 90, 8375, 400, 125, 310, Terrain.ASTEROID_BELT,  "Finnigan's Gift");
		system.addRingBand(star, "misc", "rings_ice0", 256f, 0, Color.white, 256f, 8375, 300f);
		system.addRingBand(star, "misc", "rings_ice0", 256f, 3, Color.white, 256f, 8525, 250f);

		SectorEntityToken pirateStation2 = system.addCustomEntity("finnigans", "Finnigan's Folly", "station_pirate_type", "pirates");
		pirateStation2.setCircularOrbitPointingDown(system.getEntityById("canis"), 90, 8175, 200);		
		pirateStation2.setCustomDescriptionId("finnigans");
		pirateStation2.setInteractionImage("illustrations", "pirate_station");

		PlanetAPI canis4 = system.addPlanet("laika", star, "Laika", "cryovolcanic", 240, 120, 12000, 480);

		SectorEntityToken relay = system.addCustomEntity("pack_relay", // unique id
				 "Petra Relay", // name - if null, defaultName from custom_entities.json will be used
				 "comm_relay", // type of object, defined in custom_entities.json
				 "pack"); // faction
		relay.setCircularOrbit( system.getEntityById("canis"), 2250, 12000, 480);

		JumpPointAPI jumpPoint = Global.getFactory().createJumpPoint("canis_gate", "Canis Jump-Point");
		OrbitAPI orbit = Global.getFactory().createCircularOrbit(star, 230, 12000, 480);
		jumpPoint.setOrbit(orbit);
		jumpPoint.setRelatedPlanet(canis4);
		jumpPoint.setStandardWormholeToHyperspaceVisual();
		system.addEntity(jumpPoint);
		
		SectorEntityToken station = system.addCustomEntity("port_revive", "Port Revive", "station_side00", "tritachyon");


		station.setCustomDescriptionId("station_port_revive");
		station.setCircularOrbit(star, 245, 11000, 480);

		// generates hyperspace destinations for in-system jump points
		system.autogenerateHyperspaceJumpPoints(true, true);
		
                cleanup(system);
	}
        
    void cleanup(StarSystemAPI system){
        HyperspaceTerrainPlugin plugin = (HyperspaceTerrainPlugin) Misc.getHyperspaceTerrain().getPlugin();
	NebulaEditor editor = new NebulaEditor(plugin);        
        float minRadius = plugin.getTileSize() * 2f;
        
        float radius = system.getMaxRadiusInHyperspace();
        editor.clearArc(system.getLocation().x, system.getLocation().y, 0, radius + minRadius * 0.5f, 0, 360f);
        editor.clearArc(system.getLocation().x, system.getLocation().y, 0, radius + minRadius, 0, 360f, 0.25f);	     
//        editor.clearArc(system.getLocation().x, system.getLocation().y, 0, system.getMaxRadiusInHyperspace()*1.25f, 0, 360, 0.25f);
    }	
}
