package data.scripts.world.systems;

import java.awt.Color;
import java.util.List;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.CargoAPI;
import com.fs.starfarer.api.campaign.JumpPointAPI;
import com.fs.starfarer.api.campaign.LocationAPI;
import com.fs.starfarer.api.campaign.OrbitAPI;
import com.fs.starfarer.api.campaign.PlanetAPI;
import com.fs.starfarer.api.campaign.SectorAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.StarSystemAPI;
//import com.fs.starfarer.api.campaign.CargoAPI.CrewXPLevel;
import com.fs.starfarer.api.fleet.FleetMemberType;
import com.fs.starfarer.api.impl.campaign.ids.Terrain;
import com.fs.starfarer.api.impl.campaign.terrain.BaseRingTerrain.RingParams;
import com.fs.starfarer.api.impl.campaign.terrain.AsteroidFieldTerrainPlugin.AsteroidFieldParams;
import com.fs.starfarer.api.impl.campaign.terrain.MagneticFieldTerrainPlugin.MagneticFieldParams;
import com.fs.starfarer.api.util.Misc;
import com.fs.starfarer.api.impl.campaign.terrain.HyperspaceTerrainPlugin;
import com.fs.starfarer.api.impl.campaign.procgen.NebulaEditor;

public class Ursulo {

	public void generate(SectorAPI sector) {
		
		StarSystemAPI system = sector.createStarSystem("Ursulo");
		LocationAPI hyper = Global.getSector().getHyperspace();
		
		system.setBackgroundTextureFilename("graphics/backgrounds/background6.jpg");
		
		// create the star and generate the hyperspace anchor for this system
		PlanetAPI star = system.initStar("ursulo", // unique id for this star 
										 "star_yellow", // id in planets.json
										 450f, 		// radius (in pixels at default zoom)
										 400);          // corona
		
		system.setLightColor(new Color(235, 255, 200)); // light color in entire system, affects all entities
		
		/*
		 * addPlanet() parameters:
		 * 1. Unique id for this planet (or null to have it be autogenerated)
		 * 2. What the planet orbits (orbit is always circular)
		 * 3. Name
		 * 4. Planet type id in planets.json
		 * 5. Starting angle in orbit, i.e. 0 = to the right of the star
		 * 6. Planet radius, pixels at default zoom
		 * 7. Orbit radius, pixels at default zoom
		 * 8. Days it takes to complete an orbit. 1 day = 10 seconds.
		 */
		
		
		


		PlanetAPI ur1 = system.addPlanet("rupert", star, "Rupert", "barren-bombarded", 75, 75, 1550, 60);
		//ur1.setFaction("independent");    		
		//ur1.setCustomDescriptionId("planet_ursuloI");

		PlanetAPI ur2 = system.addPlanet("winnipeg", star, "Winnipeg", "rocky_metallic", 180, 85, 3200, 64);
		//ur2.setCustomDescriptionId("planet_ursuloII");

		ur2.getSpec().setPlanetColor(new Color(195,195,215,255));
		ur2.getSpec().setAtmosphereColor(new Color(160,110,45,140));
		ur2.getSpec().setCloudColor(new Color(255,164,96,200));
		ur2.getSpec().setTilt(15);
		ur2.applySpecChanges();

		//system.addRingBand(ur2, "misc", "rings1", 256f, 2, Color.white, 256f, 250, 40f);
		//system.addRingBand(ur2, "misc", "rings1", 256f, 2, Color.white, 256f, 260, 60f);

		system.addRingBand(star, "misc", "rings_ice0", 256f, 0, Color.white, 256f, 4100, 100f);
		system.addRingBand(star, "misc", "rings_asteroids0", 256f, 1, Color.white, 256f, 4150, 220f);
		system.addAsteroidBelt(star, 90, 4200, 400, 125, 210, Terrain.ASTEROID_BELT,  "Fireworks Belt");
		system.addRingBand(star, "misc", "rings_ice0", 256f, 3, Color.white, 256f, 4225, 360f);
		system.addRingBand(star, "misc", "rings_dust0", 256f, 2, Color.white, 256f, 4300, 240f);

		SectorEntityToken ffstation = system.addCustomEntity("fireworks_factory", "Fireworks Factory", "station_pirate_type", "syndicate_asp");
		ffstation.setCircularOrbitPointingDown(system.getEntityById("ursulo"), 90, 4200, 220);		
		ffstation.setCustomDescriptionId("fireworks_factory");

		//SectorEntityToken ring = system.addTerrain(Terrain.RING, new RingParams(200 + 256, 4200, null, "Fireworks Belt"));
		//ring.setCircularOrbit(star, 0, 0, 100);

		PlanetAPI ur3 = system.addPlanet("paddington", star, "Paddington", "arid", 30, 150, 5500, 130);
		ur3.getSpec().setGlowTexture(Global.getSettings().getSpriteName("hab_glows", "volturn"));
		ur3.getSpec().setGlowColor(new Color(245,255,250,175));
		
		ur3.setFaction("syndicate_asp");ur3.setInteractionImage("illustrations", "asp_paddington");ur3.applySpecChanges();
		ur3.setCustomDescriptionId("planet_ursuloIII");

		SectorEntityToken station = system.addCustomEntity("paddington_station", "Paddington Station", "station_side04", "syndicate_asp");
		station.setCircularOrbitPointingDown(system.getEntityById("paddington"), 90, 450, 45);		
		station.setCustomDescriptionId("paddington_station");

		PlanetAPI ur3a = system.addPlanet("baloo", ur3, "Baloo", "cryovolcanic", 280, 50, 800, 30);

		ur3a.getSpec().setPlanetColor(new Color(215,255,190,255));
		ur3a.getSpec().setAtmosphereColor(new Color(160,110,45,140));
		ur3a.getSpec().setCloudColor(new Color(175,135,245,200));
		ur3a.getSpec().setTilt(10);
		ur3a.applySpecChanges();

		PlanetAPI ur4 = system.addPlanet("aloysius", star, "Aloysius", "gas_giant", 75, 260, 11000, 500);

		SectorEntityToken pirateStation = system.addCustomEntity("bear_pit", "Bear's Pit", "station_pirate_type", "pirates");
		pirateStation.setCircularOrbitPointingDown(system.getEntityById("aloysius"), 75, 400, 500);		
		pirateStation.setCustomDescriptionId("bear_pit");
		pirateStation.setInteractionImage("illustrations", "pirate_station");

			// Aloysius trojans
			SectorEntityToken aloyL4 = system.addTerrain(Terrain.ASTEROID_FIELD,
					new AsteroidFieldParams(
						500f, // min radius
						700f, // max radius
						20, // min asteroid count
						30, // max asteroid count
						4f, // min asteroid radius 
						16f, // max asteroid radius
						"Aloysius L4 Asteroids")); // null for default name
			
			SectorEntityToken aloyL5 = system.addTerrain(Terrain.ASTEROID_FIELD,
					new AsteroidFieldParams(
						500f, // min radius
						700f, // max radius
						20, // min asteroid count
						30, // max asteroid count
						4f, // min asteroid radius 
						16f, // max asteroid radius
						"Aloysius L5 Asteroids")); // null for default name
			
			aloyL4.setCircularOrbit(star, 135f, 11000, 500);
			aloyL5.setCircularOrbit(star, 15f, 11000, 500);

		//ur4.setCustomDescriptionId("planet_ursuloIV");

		ur4.getSpec().setPlanetColor(new Color(175,135,245,255));
		ur4.getSpec().setAtmosphereColor(new Color(160,110,45,140));
		ur4.getSpec().setCloudColor(new Color(195,195,215,200));
		ur4.getSpec().setTilt(20);
		ur4.applySpecChanges();

		PlanetAPI ur5 = system.addPlanet("yogi", star, "Yogi", "terran", 270, 65, 7800, 400);

		ur5.getSpec().setGlowTexture(Global.getSettings().getSpriteName("hab_glows", "volturn"));
		ur5.getSpec().setGlowColor(new Color(245,255,250,175));
		ur5.applySpecChanges();
		ur5.setFaction("independent");


		SectorEntityToken relay = system.addCustomEntity("ursulo_relay", // unique id
				 "Super-T.E.D. Relay", // name - if null, defaultName from custom_entities.json will be used
				 "comm_relay", // type of object, defined in custom_entities.json
				 "syndicate_asp"); // faction
		relay.setCircularOrbit( system.getEntityById("ursulo"), 255, 7800, 400);

		ur5.setCustomDescriptionId("planet_ursuloV");
		//ur3a.setCustomDescriptionId("planet_ursuloVI");

		JumpPointAPI jumpPoint = Global.getFactory().createJumpPoint("ursulo_gate", "Ursulo Hyper-Junction");
		OrbitAPI orbit = Global.getFactory().createCircularOrbit(star, 285, 7800, 400);
		jumpPoint.setOrbit(orbit);
		jumpPoint.setRelatedPlanet(ur5);
		jumpPoint.setStandardWormholeToHyperspaceVisual();
		system.addEntity(jumpPoint);
		
		//SectorEntityToken station = system.addOrbitalStation("diktat_cnc", a1, 45, 300, 50, "Command & Control", "sindrian_diktat");
		//initStationCargo(station);
		
		//SectorEntityToken station = system.addCustomEntity("diktat_cnc", "Command & Control", "station_side02", "sindrian_diktat");
		//station.setCircularOrbitPointingDown(system.getEntityById("sindria"), 45, 300, 50);		
//		station.setCustomDescriptionId("station_ragnar");
		
		// example of using custom visuals below
//		a1.setCustomInteractionDialogImageVisual(new InteractionDialogImageVisual("illustrations", "hull_breach", 800, 800));
//		jumpPoint.setCustomInteractionDialogImageVisual(new InteractionDialogImageVisual("illustrations", "space_wreckage", 1200, 1200));
//		station.setCustomInteractionDialogImageVisual(new InteractionDialogImageVisual("illustrations", "cargo_loading", 1200, 1200));
		
		// generates hyperspace destinations for in-system jump points
		system.autogenerateHyperspaceJumpPoints(true, true);
		
		
		/*
		DiktatPatrolSpawnPoint patrolSpawn = new DiktatPatrolSpawnPoint(sector, system, 5, 3, a1);
		system.addScript(patrolSpawn);
		for (int i = 0; i < 5; i++)
			patrolSpawn.spawnFleet();

		DiktatGarrisonSpawnPoint garrisonSpawn = new DiktatGarrisonSpawnPoint(sector, system, 30, 1, a1, a1);
		system.addScript(garrisonSpawn);
		garrisonSpawn.spawnFleet();
		
		
		system.addScript(new IndependentTraderSpawnPoint(sector, hyper, 1, 10, hyper.createToken(-6000, 2000), station));
		*/
                cleanup(system);
	}
        
    void cleanup(StarSystemAPI system){
        HyperspaceTerrainPlugin plugin = (HyperspaceTerrainPlugin) Misc.getHyperspaceTerrain().getPlugin();
	NebulaEditor editor = new NebulaEditor(plugin);        
        float minRadius = plugin.getTileSize() * 2f;
        
        float radius = system.getMaxRadiusInHyperspace();
        editor.clearArc(system.getLocation().x, system.getLocation().y, 0, radius + minRadius * 0.5f, 0, 360f);
        editor.clearArc(system.getLocation().x, system.getLocation().y, 0, radius + minRadius, 0, 360f, 0.25f);	     
//        editor.clearArc(system.getLocation().x, system.getLocation().y, 0, system.getMaxRadiusInHyperspace()*1.25f, 0, 360, 0.25f);
    }
}
