package data.scripts.world.systems;

import java.awt.Color;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.JumpPointAPI;
import com.fs.starfarer.api.campaign.LocationAPI;
import com.fs.starfarer.api.campaign.OrbitAPI;
import com.fs.starfarer.api.campaign.PlanetAPI;
import com.fs.starfarer.api.campaign.SectorAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.StarSystemAPI;
import com.fs.starfarer.api.impl.campaign.ids.Terrain;
import com.fs.starfarer.api.impl.campaign.terrain.BaseTiledTerrain;
import com.fs.starfarer.api.impl.campaign.terrain.MagneticFieldTerrainPlugin.MagneticFieldParams;
import com.fs.starfarer.api.util.Misc;

public class York {

	public void generate(SectorAPI sector) {
		
		StarSystemAPI system = sector.createStarSystem("York");
		LocationAPI hyper = Global.getSector().getHyperspace();
		
		system.setBackgroundTextureFilename("graphics/backgrounds/background6.jpg");
		
		// create the star and generate the hyperspace anchor for this system
		PlanetAPI star = system.initStar("york", // unique id for this star 
										 "star_yellow", // id in planets.json
										 400f, 150); 		// radius (in pixels at default zoom)
		
		system.setLightColor(new Color(225, 245, 200)); // light color in entire system, affects all entities
		
		/*
		 * addPlanet() parameters:
		 * 1. Unique id for this planet (or null to have it be autogenerated)
		 * 2. What the planet orbits (orbit is always circular)
		 * 3. Name
		 * 4. Planet type id in planets.json
		 * 5. Starting angle in orbit, i.e. 0 = to the right of the star
		 * 6. Planet radius, pixels at default zoom
		 * 7. Orbit radius, pixels at default zoom
		 * 8. Days it takes to complete an orbit. 1 day = 10 seconds.
		 */
		
		
		


		PlanetAPI york1 = system.addPlanet("marple", star, "Marple", "rocky_metallic", 270, 80, 1000, 120);
		//york1.setFaction("independent");    		
		//york1.setCustomDescriptionId("planet_yorkI");

		PlanetAPI york2 = system.addPlanet("appley_bridge", star, "Appley Bridge", "rocky_ice", 120, 95, 1850, 90);
		//york2.setCustomDescriptionId("planet_yorkII");



		PlanetAPI york3 = system.addPlanet("lincoln", star, "Lincoln", "terran", 120, 150, 3600, 130);
		york3.setFaction("hegemony");
		york3.setCustomDescriptionId("planet_yorkIII");
		york3.getSpec().setGlowTexture(Global.getSettings().getSpriteName("hab_glows", "volturn"));
		york3.getSpec().setGlowColor(new Color(245,255,250,75));
		york3.applySpecChanges();

		system.addRingBand(york3, "misc", "rings1", 256f, 2, Color.white, 256f, 400, 40f);
		system.addRingBand(york3, "misc", "rings1", 256f, 2, Color.white, 256f, 420, 60f);

		PlanetAPI york4 = system.addPlanet("swanage", star, "Swanage", "gas_giant", 15, 280, 12500, 450);

		//york4.setCustomDescriptionId("planet_yorkIV");

		york4.getSpec().setPlanetColor(new Color(215,225,115,255));
		york4.getSpec().setAtmosphereColor(new Color(160,185,45,140));
		york4.getSpec().setCloudColor(new Color(195,155,115,200));
		york4.getSpec().setTilt(45);
		york4.applySpecChanges();

		SectorEntityToken relay = system.addCustomEntity("york_relay", // unique id
				 "York Relay", // name - if null, defaultName from custom_entities.json will be used
				 "comm_relay", // type of object, defined in custom_entities.json
				 "hegemony"); // faction
		relay.setCircularOrbit( system.getEntityById("lincoln"), 350, 1200, 60);

		PlanetAPI york4a = system.addPlanet("dartmouth", york4, "Dartmouth", "rocky_ice", 290, 55, 800, 45);
		york4a.getSpec().setGlowTexture(Global.getSettings().getSpriteName("hab_glows", "barren"));
		york4a.getSpec().setGlowColor(new Color(245,235,250,185));
		york4a.applySpecChanges();
		york4a.setFaction("junk_pirates");

		PlanetAPI york4b = system.addPlanet("christchurch", york4, "Christchurch", "rocky_metallic", 280, 65, 1000, 55);
		york4b.getSpec().setGlowTexture(Global.getSettings().getSpriteName("hab_glows", "volturn"));
		york4b.getSpec().setGlowColor(new Color(245,255,250,125));
		york4b.applySpecChanges();
		york4b.setFaction("pack");		

		york4a.setCustomDescriptionId("planet_yorkIVa");
		york4b.setCustomDescriptionId("planet_yorkIVb");

		JumpPointAPI jumpPoint = Global.getFactory().createJumpPoint("york_gate", "York Gateway");
		OrbitAPI orbit = Global.getFactory().createCircularOrbit(york3, 0, 500, 30);
		jumpPoint.setOrbit(orbit);
		jumpPoint.setRelatedPlanet(york3);
		jumpPoint.setStandardWormholeToHyperspaceVisual();
		system.addEntity(jumpPoint);
		
		//SectorEntityToken station = system.addOrbitalStation("diktat_cnc", a1, 45, 300, 50, "Command & Control", "sindrian_diktat");
		//initStationCargo(station);
		
		SectorEntityToken station = system.addCustomEntity("lincoln_cathedral", "Lincoln Cathedral", "station_side02", "hegemony");
		station.setCircularOrbitPointingDown(system.getEntityById("lincoln"), 45, 300, 50);		
		station.setCustomDescriptionId("station_lincoln_cathedral");
		
		// example of using custom visuals below
//		a1.setCustomInteractionDialogImageVisual(new InteractionDialogImageVisual("illustrations", "hull_breach", 800, 800));
//		jumpPoint.setCustomInteractionDialogImageVisual(new InteractionDialogImageVisual("illustrations", "space_wreckage", 1200, 1200));
//		station.setCustomInteractionDialogImageVisual(new InteractionDialogImageVisual("illustrations", "cargo_loading", 1200, 1200));
		
		// generates hyperspace destinations for in-system jump points
		system.autogenerateHyperspaceJumpPoints(true, true);
		
		
		/*
		DiktatPatrolSpawnPoint patrolSpawn = new DiktatPatrolSpawnPoint(sector, system, 5, 3, a1);
		system.addScript(patrolSpawn);
		for (int i = 0; i < 5; i++)
			patrolSpawn.spawnFleet();

		DiktatGarrisonSpawnPoint garrisonSpawn = new DiktatGarrisonSpawnPoint(sector, system, 30, 1, a1, a1);
		system.addScript(garrisonSpawn);
		garrisonSpawn.spawnFleet();
		
		
		system.addScript(new IndependentTraderSpawnPoint(sector, hyper, 1, 10, hyper.createToken(-6000, 2000), station));
		*/
	}
	
}
